#include <stdarg.h>
#include <string>
#include <string.h>
#include <stdlib.h>
#include <Core/CFString.h>
#include <Core/CFLog.h>
#include <Core/CFStringList.h>

CFString::CFString()
    : m_data(NULL)
    , m_strLen(0)
    , m_utfLen(0) {
}

CFString::CFString(const CFChar* str)
    : m_data(NULL)
    , m_strLen(0)
    , m_utfLen(0) {
    create((const CFUChar*)str);
}

CFString::CFString(const CFUChar* str)
    : m_data(NULL)
    , m_strLen(0)
    , m_utfLen(0) {
    create(str);
}

CFString::CFString(const CFString& other)
    : m_data(NULL) {
    create(other.m_data);
}

CFString::~CFString() {
	if (NULL != m_data) {
        delete [] m_data;
        m_data = NULL;
	}
	m_strLen = 0;
	m_utfLen = 0;
}

CFUInt CFString::clength() {
    return m_strLen;
}

CFUInt CFString::length() {
    return m_utfLen;
}

char* CFString::toCString() {
    if (NULL != m_data) {
        return strdup((char*)m_data);
    }
    return NULL;
}

std::string CFString::toStdString() {
    if (NULL != m_data) {
        return std::string((char*)(m_data));
    }
    return std::string();
}

CFUChar* CFString::toUtf8() {
    if (NULL != m_data) {
        return (CFUChar*)strdup((char*)m_data);
    }
    return NULL;
}

CFStringList CFString::split(const CFString& splitStr) {
    CFStringList ret;
    int startPos = 0;

    while (startPos < m_utfLen) {
        int endPos = indexOf(splitStr, startPos);
        if (0 > endPos) {
            ret.append(sub(startPos, -1));
            break;
        }
        if (startPos < endPos) {
            ret.append(sub(startPos, endPos - startPos));
        }
        startPos = endPos + 1;
    }
    return ret;
}

CFString CFString::sub(CFUInt pos, CFInt len) {
    if (NULL == m_data) {
        return "";
    }
	if (0 == len) {
        return CFString();
    }
    CFUInt index = convertPosToIndex(pos);
    if (0 > len) {
        len = m_utfLen - pos;
    }

    CFInt dataLen = 6 * len + 1;
    CFUChar* strret = new CFUChar[dataLen]; // 假设所有字符都是最长的utf8编码文字
    memset(strret, 0x0, sizeof(CFUChar) * (dataLen));
    CFUChar chr;
    int i = 0;

    CFInt wordCount = 0;
    while (i < dataLen && i < m_strLen && wordCount < len) {
        int charwidth = 0;
        chr = *(m_data + i + index);
        charwidth = wordWidth(chr);

        if (1 < charwidth) {
            for (int j = 0; j < charwidth && i < dataLen && i < m_strLen; ++j, ++i) {
                strret[i] = *(m_data + i + index);
            }
        } else {
            strret[i] = chr;
            i++;
        }
        wordCount++;
    }

    CFString stringRet = CFString(strret);
    delete [] strret;
    return stringRet;
}

CFInt CFString::indexOf(CFUChar chr, int from, bool caseSensitive) {
    CFUChar strtmp[] = {chr};
    return indexOf(strtmp, from, caseSensitive);
}

CFInt CFString::indexOf(const CFString& str, int from, bool caseSensitive) {
    if (str.isEmpty() || isEmpty() || from >= m_strLen) {
        return -1;
    }

    from = convertPosToIndex(from);
    int i = 0;
    int firstIndex = -1;
    CFUChar srcChr = *m_data;
    CFUChar destChr = *str.m_data;
    bool isFound = false;
    for (i = from; i < m_strLen; ++i) {
        srcChr = *(m_data + i);
        destChr = *(str.m_data);
        firstIndex = -1;
        isFound = false;
        if (charEqual(destChr, srcChr, caseSensitive)) {
            firstIndex = i;
            isFound = true;
            for (int j = 0; j < str.m_strLen; ++j) {
                if ((firstIndex + j) >= m_strLen) {
                    isFound = false;
                    break;
                }
                srcChr = *(m_data + i + j);
                destChr = *(str.m_data + j);
                if (!charEqual(destChr, srcChr, caseSensitive)) {
                    isFound = false;
                    break;
                }
            }
            if (isFound) {
                break;
            }
        }
    }
    if (0 <= firstIndex && m_strLen > firstIndex && isFound) {
        return convertIndexToPos(firstIndex);
    }
    return -1;
}

bool CFString::isUtf8() const {
    if (NULL == m_data) {
        return false;
    }
    int i = 0;
    CFUChar chr;
    int charwidth = 0;
    for (i = 0; i < m_strLen; i = i + charwidth) {
        chr = *(m_data + i);
        charwidth = wordWidth(chr);
        if (1 < charwidth) {
            return true;
        }
    }
    return false;
}

bool CFString::isEmpty() const {
    return (m_strLen == 0 || NULL == m_data || (*(m_data) == '\0'));
}

bool CFString::equal(const CFString& other) {
    return (0 == compare(other));
}

CFInt CFString::compare(const CFString& other) {
    if (NULL == m_data || NULL == other.m_data) {
        return -1;
    }
    int compLength = 0;
    if ((m_strLen != other.m_strLen)
        || (m_utfLen != other.m_utfLen)) {
        compLength = (m_strLen < other.m_strLen) ? m_strLen : other.m_strLen;
    } else {
    	    compLength = m_strLen;
    }

    int ret = memcmp(m_data, other.m_data, compLength);
    if (0 == ret) {
        if (m_strLen != other.m_strLen) {
            return (m_strLen - other.m_strLen);
        }

        if (m_utfLen != other.m_utfLen) {
            return (m_utfLen - other.m_utfLen);
        }
    }
    return ret;
}

void CFString::display() {
	CFLog::d("CFString", "LINE: %d, FUNC: %s -----> data(%s), strlen(%d), utflen(%d)", __LINE__, __PRETTY_FUNCTION__, m_data, m_strLen, m_utfLen);
}

CFUInt CFString::getUTF8length () {
    if (NULL == m_data) {
        return 0;
    }
    int i = 0;
    CFUChar chr;
    int outLen = 0;
    int charwidth = 0;
    for (i = 0; i < m_strLen; i = i + charwidth) {
        chr = *(m_data + i);
        charwidth = wordWidth(chr);
        outLen++;
    }
    return outLen;
}

void CFString::create(const CFUChar* strdata) {
    if (NULL == strdata) {
        return;
    }
    if (NULL != m_data) {
        delete [] m_data;
        m_data = NULL;
    }

    m_strLen = dataLength(strdata);
    m_data = new CFUChar[m_strLen + 1];
    memset(m_data, 0x0, sizeof(CFUChar) * (m_strLen + 1));
    memcpy(m_data, strdata, sizeof(CFUChar) * (m_strLen));
    m_utfLen = getUTF8length();
}

CFUInt CFString::convertPosToIndex(CFUInt pos) {
    if (NULL == m_data) {
        return 0;
    }
    int i = 0;
    CFUChar chr;
    int chrPos = 0;
    int charwidth = 0;
    for (i = 0; i < m_strLen; i = i + charwidth) {
    	    if (chrPos >= pos) {
            return i;
    	    }
        chr = *(m_data + i);
        charwidth = wordWidth(chr);
        chrPos++;
    }
    return i;
}

CFInt CFString::convertIndexToPos(CFInt index) {
    if (NULL == m_data || 0 > index) {
        return -1;
    }
    int i = 0;
    CFInt wordCount = 0;
    CFUChar chr;
    while (i < m_strLen && i < index) {
        chr = *(m_data + i);
        i += wordWidth(chr);
        wordCount++;
    }
    return wordCount;
}

CFUInt CFString::dataLength(const CFUChar* strdata) {
    if (NULL == strdata) {
        return 0;
    }
    int cnt = 0;
    CFUChar chr = *(strdata + cnt);
    CFUInt charwidth = 1;
    while (0 != chr) {
        charwidth = wordWidth(chr);
        cnt += charwidth;
        chr = *(strdata + cnt);
    }
    return cnt;
}

CFUInt CFString::wordWidth(CFUChar chr) const {
    CFUInt charwidth = 1;
    if (0 != (chr & 0x80)) {
        if (chr >= 0xFC && chr <= 0xFD) {
            charwidth = 6;
        } else if (chr >= 0xF8) {
            charwidth = 5;
        } else if (chr >= 0xF0) {
            charwidth = 4;
        } else if (chr >= 0xE0) {
            charwidth = 3;
        } else if (chr >= 0xC0) {
            charwidth = 2;
        } else {
            charwidth = 1;
        }
    } else {
        charwidth = 1;
    }
    return charwidth;
}

bool CFString::charEqual(CFUChar destChr, CFUChar srcChr, bool caseSensitive) {
    int charOffSet = caseSensitive ? 0 : 32;
    if ('a' <= destChr && 'z' >= destChr) {
        if (srcChr == destChr || srcChr == destChr - charOffSet) {
            return true;
        }
    } else if ('A' <= destChr && 'Z' >= destChr) {
        if (srcChr == destChr || srcChr == destChr + charOffSet) {
            return true;
        }
    } else {
        if (destChr == srcChr) {
            return true;
        }
    }
    return false;
}

CFString operator+(const CFString& str1, const CFString& str2) {
    if (NULL == str1.m_data || NULL == str2.m_data) {
        return CFString();
    }
    CFInt fullLength = str1.m_strLen + str2.m_strLen + 1;
    CFUChar* stringRet = new CFUChar[fullLength];
    CFInt i = 0;
    memset(stringRet, 0x0, sizeof(CFUChar) * fullLength);

    memccpy(stringRet, str1.m_data, 0, sizeof(CFUChar) * str1.m_strLen);
    memccpy(stringRet + sizeof(CFUChar) * str1.m_strLen, str2.m_data, 0, sizeof(CFUChar) * str2.m_strLen);

    CFString ret = CFString(stringRet);
    delete [] stringRet;
    return ret;
}

CFString& CFString::operator=(const CFString& str1) {
    if (NULL == str1.m_data) {
        return *this;
    }
    create(str1.m_data);
    return *this;
}

CFString& CFString::operator+=(const CFString& str1) {
    if (NULL == str1.m_data) {
        return *this;
    }
    CFString tmp = *(this) + str1;
    create(tmp.m_data);
    return *this;
}

CFString operator+(const CFString& str1, const CFUChar* str2) {
    return CFString(str1 + CFString(str2));
}

CFString operator+(const CFUChar* str1, const CFString& str2) {
    return CFString(CFString(str1) + str2);
}

CFString operator+(const CFString& str1, const CFChar* str2) {
    return CFString(str1 + CFString(str2));
}

CFString operator+(const CFChar* str1, const CFString& str2) {
    return CFString(CFString(str1) + str2);
}

// CFString operator+(const CFUChar* str1, const CFUChar* str2) {
//     return CFString(str1) + CFString(str2);
// }

bool CFString::operator==(const CFString& other) {
    return equal(other);
}

bool CFString::operator!=(const CFString& other) {
    return (!equal(other));
}

bool CFString::operator>(const CFString& other) {
    return (0 < compare(other));
}

bool CFString::operator>=(const CFString& other) {
    return (0 <= compare(other));
}

bool CFString::operator<(const CFString& other) {
    return (0 > compare(other));
}

bool CFString::operator<=(const CFString& other) {
    return (0 >= compare(other));
}
