#include <Support/CFLunar.h>
#include <string.h>
#include <time.h>

extern CFUInt16 gLunarMonthDay[];
extern CFUInt8 gLunarMonth[];
extern CFUInt8 gLunarHolDay[];

const CFUInt16 LUNAR_START_YEAR = 1901;
const CFUInt16 LUNAR_END_YEAR = 2050;

#define LOWORD(l) ((CFUInt8)(l))
#define HIWORD(l) ((CFUInt8)(((unsigned long)(l) >> 16) & 0xFFFF))
#define MAKELONG(a, b) ((long)(((CFUInt8)(a)) | ((unsigned long)((CFUInt8)(b))) << 16))

void CFLunar::l_initData() {
    struct tm *tm;
    time_t t = time(NULL);
    tm = localtime(&t);
    m_year = tm->tm_year + 1900;
    m_month = tm->tm_mon + 1;
    m_day = tm->tm_mday;
}

CFLunar::CFLunar(CFUInt16 iYear, CFUInt16 iMonth, CFUInt16 iDay) {
    if (!setDate(iYear, iMonth, iDay))
        l_initData();
}

CFLunar::CFLunar() { 
    l_initData(); 
}

long CFLunar::calcDateDiff(CFUInt16 iEndYear, CFUInt16 iEndMonth,
                         CFUInt16 iEndDay, CFUInt16 iStartYear,
                         CFUInt16 iStartMonth, CFUInt16 iStartDay) {
    CFUInt16 monthday[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
    long iDiffDays = (iEndYear - iStartYear) * 365;
    iDiffDays += (iEndYear - 1) / 4 - (iStartYear - 1) / 4;
    iDiffDays -= ((iEndYear - 1) / 100 - (iStartYear - 1) / 100);
    iDiffDays += (iEndYear - 1) / 400 - (iStartYear - 1) / 400;
    iDiffDays += monthday[iEndMonth - 1] + (isLeapYear(iEndYear) && iEndMonth > 2 ? 1 : 0);
    iDiffDays += iEndDay;
    iDiffDays -= (monthday[iStartMonth - 1] + 
		    (isLeapYear(iStartYear) && iStartMonth > 2 ? 1 : 0));
    iDiffDays -= iStartDay;
    return iDiffDays;
}

void CFLunar::l_calcLunarDate(CFUInt16 &iYear, CFUInt16 &iMonth, 
		        CFUInt16 &iDay, long iSpanDays) {
    if (iSpanDays < 49) {
        iYear = LUNAR_START_YEAR - 1;
        if (iSpanDays < 19) {
            iMonth = 11;
            iDay = 11 + (CFUInt16)(iSpanDays);
        } else {
            iMonth = 12;
            iDay = (CFUInt16)(iSpanDays)-18;
        }
        return;
    }

    iSpanDays -= 49;
    iYear = LUNAR_START_YEAR;
    iMonth = 1;
    iDay = 1;

    long tmp = lunarYearDays(iYear);
    while (iSpanDays >= tmp) {
        iSpanDays -= tmp;
        tmp = lunarYearDays(++iYear);
    }

    tmp = LOWORD(lunarMonthDays(iYear, iMonth));
    while (iSpanDays >= tmp) {
        iSpanDays -= tmp;
        if (iMonth == getLeapMonth(iYear)) {
            tmp = HIWORD(lunarMonthDays(iYear, iMonth));
            if (iSpanDays < tmp)
                break;
            iSpanDays -= tmp;
        }
        tmp = LOWORD(lunarMonthDays(iYear, ++iMonth));
    }

    iDay += (CFUInt16)(iSpanDays);
}

void CFLunar::getLunarDate(CFUInt16 iYear, CFUInt16 iMonth, 
		CFUInt16 iDay, CFUInt16 &iLunarYear,
		CFUInt16 &iLunarMonth, CFUInt16 &iLunarDay) {
    l_calcLunarDate(iLunarYear, iLunarMonth, iLunarDay, calcDateDiff(iYear, iMonth, iDay));
}

bool CFLunar::getSolarTerm (CFUInt16 iYear, CFUInt16 iMonth, CFUInt16 iDay, char *pBuffer) {
    CFUInt16 solarTerm;
    solarTerm = l_getLunarHolDay (iYear, iMonth, iDay);
    if (0 < solarTerm && solarTerm < 24) {
        char solarTerms[] = "小寒大寒立春雨水惊蛰春分清明谷雨立夏小满芒种夏至小暑大暑立秋处暑白露秋分寒露霜降立冬小雪大雪冬至";
        short wide = strlen (solarTerms)/48;
        memcpy (pBuffer, solarTerms + (solarTerm - 1) * 2 * wide, 2 * wide);
        pBuffer[2 * wide] = '\0';
        return true;
    }
    return false;
}

CFUInt16 CFLunar::l_getLunarHolDay(CFUInt16 iYear, CFUInt16 iMonth, 
		CFUInt16 iDay) {
    CFUInt8 &flag = gLunarHolDay[(iYear - LUNAR_START_YEAR) * 12 + iMonth - 1];
    CFUInt16 day;
    if (iDay < 15)
        day = 15 - ((flag >> 4) & 0x0f);
    else
        day = ((flag)&0x0f) + 15;
    if (iDay == day)
        return (iMonth - 1) * 2 + (iDay > 15 ? 1 : 0) + 1;
    else
        return 0;
}

void CFLunar::getDate(CFUInt16 &iYear, 
                    CFUInt16 &iMonth,
                    CFUInt16 &iDay) {
    iYear = m_year;
    iMonth = m_month;
    iDay = m_day;
}

int CFLunar::setDate(CFUInt16 iYear, 
                   CFUInt16 iMonth,
                   CFUInt16 iDay) {
    if (iYear < LUNAR_START_YEAR || iYear > LUNAR_END_YEAR || iMonth < 1 || iMonth > 12)
        return false;

    if (iDay < 1 || iDay > monthDays(iYear, iMonth))
        return false;

    m_year = iYear;
    m_month = iMonth;
    m_day = iDay;

    return true;
}

CFUInt16 CFLunar::weekDay(CFUInt16 iYear, 
		              CFUInt16 iMonth,
                              CFUInt16 iDay) {
    CFUInt16 monthday[] = {0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5};
    CFUInt16 iDays = 
	    (iYear - 1) % 7 + (iYear - 1) / 4 - (iYear - 1) / 100 + (iYear - 1) / 400;
    iDays += (monthday[iMonth - 1] + iDay);
    if (isLeapYear(iYear) && iMonth > 2)
        iDays++;
  
    return iDays % 7;
}

void CFLunar::formatWeekDay (CFUInt16 iYear, CFUInt16 iMonth, 
			CFUInt16 iDay, char *pBuffer) {
    const char *week = "星期";
    char weekday[] = "日一二三四五六";
    short wide = strlen (weekday)/7;
  
    CFUInt16 day = weekDay (iYear, iMonth, iDay);
    memcpy (pBuffer, week, 2 * wide);
    memcpy (pBuffer + 2 * wide, weekday + day * wide, wide);
    pBuffer[3 * wide] = '\0'; 
}

CFUInt16 CFLunar::monthDays(CFUInt16 iYear, CFUInt16 iMonth) {
    switch (iMonth) {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:
            return 31;

        case 4:
        case 6:
        case 9:
        case 11:
            return 30;

        case 2:
            if (isLeapYear(iYear))
                return 29;
            else
                return 28;
    }
    return 0;
}

CFUInt16 CFLunar::getLeapMonth(CFUInt16 iLunarYear) {
    CFUInt8 &flag = gLunarMonth[(iLunarYear - LUNAR_START_YEAR) / 2];
    return (iLunarYear - LUNAR_START_YEAR) % 2 ? flag & 0x0f : flag >> 4;
}

bool CFLunar::getFormatLeapMonth (CFUInt16 iLunarYear, char *pBuffer) {
    CFUInt16 leapMonth = getLeapMonth (iLunarYear);

    if (!leapMonth) 
        return false;

    char month[] = "正二三四五六七八九十";
    short wide = strlen (month)/10;
    strcpy (pBuffer, "闰");
    if (0 < leapMonth && leapMonth <= 10) {
        memcpy (pBuffer + wide, month + (leapMonth - 1) * wide, wide);
    } else if (leapMonth == 11) {
        strcpy (pBuffer + wide, "十一");
    } else if (leapMonth == 12)
        strcpy (pBuffer + wide, "十二");
    strcpy (pBuffer + wide * 2, "月");

    return true;
}

long CFLunar::lunarMonthDays(CFUInt16 iLunarYear,
                           CFUInt16 iLunarMonth) {
    if (iLunarYear < LUNAR_START_YEAR)
        return 30L;

    CFUInt16 height = 0, low = 29;
    int iBit = 16 - iLunarMonth;

    if (iLunarMonth > getLeapMonth(iLunarYear) && getLeapMonth(iLunarYear))
        iBit--;

    if (gLunarMonthDay[iLunarYear - LUNAR_START_YEAR] & (1 << iBit))
        low++;

    if (iLunarMonth == getLeapMonth(iLunarYear))
        if (gLunarMonthDay[iLunarYear - LUNAR_START_YEAR] & (1 << (iBit - 1)))
            height = 30;
        else
            height = 29;

    return MAKELONG(low, height);
}

CFUInt16 CFLunar::lunarYearDays(CFUInt16 iLunarYear) {
    CFUInt16 days = 0;
    for (CFUInt16 i = 1; i <= 12; i++) {
        long tmp = lunarMonthDays(iLunarYear, i);
        days += HIWORD(tmp);
        days += LOWORD(tmp);
    }
    return days;
}

void CFLunar::formatLunarYear(CFUInt16 iLunarYear, char *pBuffer) {
    char szText1[] = "甲乙丙丁戊己庚辛壬癸";
    char szText2[] = "子丑寅卯辰巳午未申酉戌亥";
    char szText3[] = "鼠牛虎免龙蛇马羊猴鸡狗猪";

    short WIDE = sizeof(szText1) / 10;

    memcpy(pBuffer, szText1 + ((iLunarYear - 4) % 10) * WIDE, WIDE);
    memcpy(pBuffer + WIDE, szText2 + ((iLunarYear - 4) % 12) * WIDE, WIDE);
    pBuffer[WIDE * 2] = ' ';
    memcpy(pBuffer + WIDE * 2 + 1, szText3 + ((iLunarYear - 4) % 12) * WIDE, WIDE);
    strcpy(pBuffer + WIDE * 3 + 1, "年");
}

void CFLunar::formatLunarMonth(CFUInt16 iLunarMonth, char *pBuffer) {
    char szText[] = "正二三四五六七八九十";

    short WIDE = sizeof(szText) / 10;

    if (iLunarMonth <= 10) {
        memcpy(pBuffer, szText + (iLunarMonth - 1) * WIDE, WIDE);
        strcpy(pBuffer + WIDE, "月");
        return;
    }
    if (iLunarMonth == 11)
        strcpy(pBuffer, "十一");
    else
        strcpy(pBuffer, "十二");
    strcpy(pBuffer + WIDE * 2, "月");
}

void CFLunar::formatLunarDay(CFUInt16 iLunarDay, char *pBuffer) {
    char szText1[] = "初十廿三";
    char szText2[] = "一二三四五六七八九十";

    short WIDE = sizeof(szText1) / 4;

    if (iLunarDay != 20 && iLunarDay != 30) {
        memcpy(pBuffer, szText1 + (iLunarDay - 1) / 10 * WIDE, WIDE);
        memcpy(pBuffer + WIDE, szText2 + ((iLunarDay - 1) % 10) * WIDE, WIDE);
        pBuffer[WIDE * 2] = '\0';
    } else {
        memcpy(pBuffer, szText1 + iLunarDay / 10 * WIDE, WIDE);
        strcpy(pBuffer + WIDE, szText2 + 9 * WIDE);
    }
}

void CFLunar::getFormatLunarDate (CFUInt16 iYear, CFUInt16 iMonth, CFUInt16 iDay, char *pBuffer) {
    CFUInt16 lunarYear, lunarMonth, lunarDay;
    char formatYear[30], formatMonth[15], formatDay[15];
    
    getLunarDate (iYear, iMonth, iDay, lunarYear, lunarMonth, lunarDay);
    formatLunarYear (lunarYear, formatYear);
    formatLunarMonth (lunarMonth, formatMonth);
    formatLunarDay (lunarDay, formatDay);

    memcpy (pBuffer, formatYear, strlen (formatYear));
    pBuffer[strlen (formatYear)] =  ' ';
    memcpy (pBuffer + strlen (formatYear) + 1, formatMonth, strlen (formatMonth));
    pBuffer[strlen (formatYear) + strlen (formatMonth) + 1] = ' ';
    strcpy (pBuffer + strlen (formatYear) + strlen (formatMonth) + 2, formatDay);
}

CFUInt16 gLunarMonthDay[] = {
    /* 1901.1.1 -- 2050.12.31 */
    0X4ae0, 0Xa570, 0X5268, 0Xd260, 0Xd950,
    0X6aa8, 0X56a0, 0X9ad0, 0X4ae8, 0X4ae0,
    0Xa4d8, 0Xa4d0, 0Xd250, 0Xd548, 0Xb550,
    0X56a0, 0X96d0, 0X95b0, 0X49b8, 0X49b0,
    0Xa4b0, 0Xb258, 0X6a50, 0X6d40, 0Xada8,
    0X2b60, 0X9570, 0X4978, 0X4970, 0X64b0,
    0Xd4a0, 0Xea50, 0X6d48, 0X5ad0, 0X2b60,
    0X9370, 0X92e0, 0Xc968, 0Xc950, 0Xd4a0,
    0Xda50, 0Xb550, 0X56a0, 0Xaad8, 0X25d0,
    0X92d0, 0Xc958, 0Xa950, 0Xb4a8, 0X6ca0, // 1950
    0Xb550, 0X55a8, 0X4da0, 0Xa5b0, 0X52b8,
    0X52b0, 0Xa950, 0Xe950, 0X6aa0, 0Xad50,
    0Xab50, 0X4b60, 0Xa570, 0Xa570, 0X5260,
    0Xe930, 0Xd950, 0X5aa8, 0X56a0, 0X96d0,
    0X4ae8, 0X4ad0, 0Xa4d0, 0Xd268, 0Xd250,
    0Xd528, 0Xb540, 0Xb6a0, 0X96d0, 0X95b0,
    0X49b0, 0Xa4b8, 0Xa4b0, 0Xb258, 0X6a50,
    0X6d40, 0Xada0, 0Xab60, 0X9370, 0X4978,
    0X4970, 0X64b0, 0X6a50, 0Xea50, 0X6b28,
    0X5ac0, 0Xab60, 0X9368, 0X92e0, 0Xc960, // 2000
    0Xd4a8, 0Xd4a0, 0Xda50, 0X5aa8, 0X56a0,
    0Xaad8, 0X25d0, 0X92d0, 0Xc958, 0Xa950,
    0Xb4a0, 0Xb550, 0Xb550, 0X55a8, 0X4ba0,
    0Xa5b0, 0X52b8, 0X52b0, 0Xa930, 0X74a8,
    0X6aa0, 0Xad50, 0X4da8, 0X4b60, 0X9570,
    0Xa4e0, 0Xd260, 0Xe930, 0Xd530, 0X5aa0,
    0X6b50, 0X96d0, 0X4ae8, 0X4ad0, 0Xa4d0,
    0Xd258, 0Xd250, 0Xd520, 0Xdaa0, 0Xb5a0,
    0X56d0, 0X4ad8, 0X49b0, 0Xa4b8, 0Xa4b0,
    0Xaa50, 0Xb528, 0X6d20, 0Xada0, 0X55b0, // 2050

};

CFUInt8 gLunarMonth[] = {
    0X00, 0X50, 0X04, 0X00, 0X20, // 1910
    0X60, 0X05, 0X00, 0X20, 0X70, // 1920
    0X05, 0X00, 0X40, 0X02, 0X06, // 1930
    0X00, 0X50, 0X03, 0X07, 0X00, // 1940
    0X60, 0X04, 0X00, 0X20, 0X70, // 1950
    0X05, 0X00, 0X30, 0X80, 0X06, // 1960
    0X00, 0X40, 0X03, 0X07, 0X00, // 1970
    0X50, 0X04, 0X08, 0X00, 0X60, // 1980
    0X04, 0X0a, 0X00, 0X60, 0X05, // 1990
    0X00, 0X30, 0X80, 0X05, 0X00, // 2000
    0X40, 0X02, 0X07, 0X00, 0X50, // 2010
    0X04, 0X09, 0X00, 0X60, 0X04, // 2020
    0X00, 0X20, 0X60, 0X05, 0X00, // 2030
    0X30, 0Xb0, 0X06, 0X00, 0X50, // 2040
    0X02, 0X07, 0X00, 0X50, 0X03  // 2050
};

CFUInt8 gLunarHolDay[] = {
    /* 1901 -- 2050 */
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X87, 0X96, 0X87, 0X87,
    0X79, 0X69, 0X69, 0X69, 0X78, 0X78,
    0X86, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X87, 0X96, 0X87, 0X87,
    0X79, 0X69, 0X69, 0X69, 0X78, 0X78,
    0X86, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X87, 0X96, 0X87, 0X87,
    0X79, 0X69, 0X69, 0X69, 0X78, 0X78,
    0X86, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X95, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X97, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X69, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA6, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA5, 0X97, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X69, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X69, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X87,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0X96, 0X96, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X77,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0XA6, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X79, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X77,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X79, 0X79, 0X79, 0X69, 0X78, 0X78,
    0X96, 0XA5, 0XA6, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X68, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA5, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X96, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X79, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB5, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X97, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X88, 0X89, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X78, 0X87,
    0X96, 0XB4, 0X96, 0XA6, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X77,
    0X96, 0XA4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X78, 0X79, 0X78, 0X69, 0X78, 0X77,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0XA6, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X79, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB4, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X86, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X76, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X69, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X78, 0X87,
    0X96, 0XB4, 0XA5, 0XB5, 0XA5, 0XA6,
    0X87, 0X88, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0X96, 0XA5, 0X96, 0X97,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X87, 0X88, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0XA6, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB4, 0XA5, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X79, 0X77, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0X96, 0X96,
    0X88, 0X78, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X78, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XA5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA5, 0XA6,
    0X87, 0X88, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X88, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X88, 0X87, 0X96,
    0XA5, 0XC3, 0XA5, 0XB4, 0XA5, 0XA6,
    0X87, 0X88, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XB3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X88, 0X78, 0X87, 0X87,
    0XA5, 0XB4, 0X96, 0XA5, 0XA6, 0X96,
    0X88, 0X88, 0X78, 0X78, 0X87, 0X87,
    0X95, 0XB4, 0XA5, 0XB4, 0XA5, 0XA5,
    0X97, 0X87, 0X87, 0X88, 0X86, 0X96,
    0XA4, 0XC3, 0XA5, 0XA5, 0XA5, 0XA6,
    0X97, 0X87, 0X87, 0X78, 0X87, 0X86,
    0XA5, 0XC3, 0XA5, 0XB5, 0XA6, 0XA6,
    0X87, 0X88, 0X78, 0X78, 0X87, 0X87
};
